/***********************************************************************************************/
/************************************** control.c **********************************************/
/***********************************************************************************************/

#define __CONTROL
#include "control.h"

/***********************************************************************************************/
/*********************************** Read Encoders and All ADCS ********************************/
/***********************************************************************************************/ 
void read_sensors(){

   char adcstr[300];

   /* servotogo adc reads */
	stgmReadADCs(ADC_CHANNELS, raw_STG_ADC_data);										/* 210 us operation for eight channels */
	if ((stg_adc_timeout_errors > 0) && (stg_adc_timeout_errors != stg_adc_timeout_printed_value)){
		stg_adc_timeout_printed_value = stg_adc_timeout_errors;
		sprintf(adcstr,"STG ADC Total Timeout Errors: %d",stg_adc_timeout_errors);
		PrintErrorMsg(adcstr);
	}

	/* servotogo encoder reads */
	stgReadEncoders(raw_STG_ENC_data.cnts); 																		/* 22 us operation for eight channels */

	/* digital inputs */
	if(stgReadDIO(MY_TTL_IN_01_DIO, PORT_A) == TRUE){
		/* react to your knowledge of digital input MY_TTL_IN_01_DIO on port A */
	}
	else{
		/* react to your knowledge of digital input MY_TTL_IN_01_DIO on port A */
	}

}

/***********************************************************************************************/
/************************** Selectively Enable/Disable Motor Amplifiers ************************/
/***********************************************************************************************/
void enable(unsigned char bitmap){
	int i = 0;
   for(i=0;i<8;i++) stgWriteDIO(i,(bitmap & (1 << i)) >> i,PORT_D);
   PrintErrorMsg("Enabling - Disabling");
}

/***********************************************************************************************/
/********************************* Bring all DAC outputs to Zero *******************************/
/***********************************************************************************************/ 
void zero_outputs(){
	int i = 0;
   for(i=0;i<8;i++) stgWriteDAC(i ,(short)(409.6*0.0));
}

/***********************************************************************************************/
/************************************* Intepret Sensor Data ************************************/
/***********************************************************************************************/ 
void interpret_sensor_data(){

   /* ---------------------- first interpret the encoder data ---------------------- */
	/* increment previous values */	
	   axis_01_p_pos = axis_01_c_pos;

   /* convert counts to radians */
   	axis_01_c_pos = TWO_PI*(double)(raw_STG_ENC_data.cnts[AXIS_01_ENC].Long)/AXIS_01_CNTS_REV - AXIS_01_RADS_OFFSET;

	/* bound position within +- PI */
		while (axis_01_c_pos >  PI) axis_01_c_pos -= TWO_PI;
		while (axis_01_c_pos < -PI) axis_01_c_pos += TWO_PI;

	/* compute the finite difference velocity */	
	   axis_01_c_vel = (axis_01_c_pos - axis_01_p_pos)*DT_INV;	
	
	/* check and correct for velocity skips due to PI to -PI wrap-around */
	   if (axis_01_c_vel < -PI*DT_INV) axis_01_c_vel = ((axis_01_c_pos + TWO_PI)- axis_01_p_pos)*DT_INV;
      if (axis_01_c_vel > PI*DT_INV)  axis_01_c_vel = (axis_01_c_pos - (axis_01_p_pos+TWO_PI))*DT_INV;
	
	/* second order software filter */
	   axis_01_f_vel = Filter(&axis_01_vel_filter,axis_01_c_vel);	

   /* ---------------------- now interpret the analog data ---------------------- */
	/* increment previous values */	
	   chan_01_p_pos = chan_01_c_pos;

   /* convert volts to your units */
   	chan_01_c_pos = ADC_01_UNITS_PER_VOLT*(raw_STG_ADC_data[CHAN_01_ADC] - ADC_01_VOLTS_OFFSET);

	/* compute the finite difference velocity */	
	   chan_01_c_vel = (axis_01_c_pos - axis_01_p_pos)*DT_INV;	

	/* second order software filter */
	   chan_01_f_vel = Filter(&chan_01_vel_filter,chan_01_c_vel);	
}

/***********************************************************************************************/
/************************************* Compute Outputs *****************************************/
/***********************************************************************************************/ 
void compute_outputs(){

   /* run your PID type controllers here and come up with a voltage command to send the dacs */
   /* I'm just putting a sinusoid here for something interesting */
      axis_01_cmd_voltage =  sinewave(20, 2.5, 1.0);

   /* check limits */
      if (axis_01_cmd_voltage > AXIS_01_DAC_VOLTAGE_LIMIT)  axis_01_cmd_voltage = AXIS_01_DAC_VOLTAGE_LIMIT;
      if (axis_01_cmd_voltage < -AXIS_01_DAC_VOLTAGE_LIMIT) axis_01_cmd_voltage = -AXIS_01_DAC_VOLTAGE_LIMIT;

   /* write output to DAC (409.6 counts per volt) */
      stgWriteDAC(AXIS_01_DAC ,(short)(409.6*axis_01_cmd_voltage));

}