/***********************************************************************************************/
/*************************************** control.h *********************************************/
/***********************************************************************************************/

#ifndef CONTROL_H
#define CONTROL_H

/***********************************************************************************************/
/************************************ Standard Headers *****************************************/
/***********************************************************************************************/

/***********************************************************************************************/
/************************************ My Headers ***********************************************/
/***********************************************************************************************/
#ifdef __CONTROL
#include "mydefines.h"				/* macros */
#include "gui.h"						/* print messages */
#include "command.h"					/* "MAIN_STATUS" parameter and "DT" */
#include "servotogo.h"				/* "TYPE_ENC" definition and function calls */
#include "ericmath.h"
#endif

#include "filter.h"

/* which digital enable voltage maps to which amplifier (should use port D for outputs)*/
#define AXIS_01_DIO_ENB			   0x00
#define AXIS_02_DIO_ENB			   0x01

/* which digital input maps to which input (should only use ports A,B, or C, not D for inputs */
#define MY_TTL_IN_01_DIO     		0x00
#define MY_TTL_IN_02_DIO     		0x01

/* amplifier DAC defines */
#define AXIS_01_DAC   			   0x00	// which physical motor amplifier maps to the servotogo dacs
#define AXIS_02_DAC   			   0x01
#define AXIS_01_DAC_VOLTAGE_LIMIT 3.2  // whatever you desire to protect your amplifier and motor
#define AXIS_02_DAC_VOLTAGE_LIMIT 3.2

/* encoder defines */
#define AXIS_01_ENC   			   0x00	// which physical enc maps to the servotogo enc
#define AXIS_02_ENC   			   0x01
#define AXIS_01_CNTS_REV         8192  // counts per revolution
#define AXIS_02_CNTS_REV         8192  // counts per revolution
#define AXIS_01_RADS_OFFSET      0.0   // nominal offset in radians
#define AXIS_02_RADS_OFFSET      0.0   // nominal offset in radians

/* sensor ADC defines */
#define CHAN_01_ADC   			   0x00	// which physical sensor maps to the servotogo adcs
#define CHAN_02_ADC   			   0x01
#define ADC_01_UNITS_PER_VOLT    1.0  // your units per volt
#define ADC_02_UNITS_PER_VOLT    1.0  // your units per volt
#define ADC_01_VOLTS_OFFSET      0.0  // nominal offset in volts
#define ADC_02_VOLTS_OFFSET      0.0  // nominal offset in volts

/* filter parameters */
#define AXIS_01_FILTER_CUTOFF    50  // filter cutoff in HZ
#define AXIS_01_FILTER_ZETA      0.707 // filter damping ratio (should be around 0.707 = critically damped)
#define CHAN_01_FILTER_CUTOFF    50  // filter cutoff in HZ
#define CHAN_01_FILTER_ZETA      0.707 // filter damping ratio (should be around 0.707 = critically damped)

/* controller gains */
#define AXIS_01_P_GAIN_INIT      1.1   //proportional gain
#define AXIS_01_I_GAIN_INIT      1.2   //integral gain
#define AXIS_01_D_GAIN_INIT      1.3   //derivative gain
#define P_GAIN_INCREMENT         0.1
#define I_GAIN_INCREMENT         0.1
#define D_GAIN_INCREMENT         0.1              

void enable(unsigned char bitmap);
void zero_outputs();
void read_sensors();
void interpret_sensor_data();
void compute_outputs();

#ifdef __MAIN
#define EXTERN
double axis_01_offset = AXIS_01_RADS_OFFSET;
double chan_01_offset = ADC_01_VOLTS_OFFSET;
double axis_01_p_gain = AXIS_01_P_GAIN_INIT;
double axis_01_i_gain = AXIS_01_I_GAIN_INIT;
double axis_01_d_gain = AXIS_01_D_GAIN_INIT;
#else
#define EXTERN extern
extern double axis_01_offset;
extern double chan_01_offset;
extern double axis_01_p_gain;
extern double axis_01_i_gain;
extern double axis_01_d_gain;
#endif

EXTERN double axis_01_c_pos;  // current position in radians
EXTERN double axis_01_p_pos;  // previous position in radians
EXTERN double axis_01_c_vel;  // current velocity in radians per second
EXTERN double axis_01_f_vel;  // filtered velocity in radians per second
EXTERN double axis_01_cmd_voltage; // command voltage for axis 01 dacs

EXTERN double chan_01_c_pos;  // current position in units
EXTERN double chan_01_p_pos;  // previous position in units
EXTERN double chan_01_c_vel;  // current velocity in units per second
EXTERN double chan_01_f_vel;  // filtered velocity in units per second

EXTERN FILTER axis_01_vel_filter;
EXTERN FILTER chan_01_vel_filter;

#endif // end of CONTROL_H
