% example command line looks like...
% "demo_data_parsing('098-12-02-41.dat',5,1,1,2000.9,1.533181)"
function [] = demo_data_parsing(fname,COLUMNS,DECIMATION,STARTPT,FREQ,CPS)

close all

faddress = strcat('ftp://root:yan.isa@192.168.1.2/root/interrupt/archive/',fname);
urlwrite(faddress,fname);
fid = fopen(fname);

[A,count] = fread(fid,[COLUMNS,inf],'float32');

FACT = 1e-03/CPS;
dt = 1.0/FREQ;
time = 0:dt:(dt*count/COLUMNS-dt);

my_time = time(STARTPT:length(time))*DECIMATION;

my_computation_time = FACT*A(1,STARTPT:length(time));
my_timestep = FACT*A(2,STARTPT:length(time));

my_variable_01 = A(3,STARTPT:length(time));
my_variable_02 = A(4,STARTPT:length(time));
my_variable_03 = A(5,STARTPT:length(time));

% Figure 1: a plot to validate the uniform timestep and the cpu workload of your
% control thread
figure(1)
subplot(2,2,1)
plot(my_time,my_computation_time,'o')
ylabel('tick timing (\musec)')
xlabel('time (sec)')
title_str = strcat('Computation Time, mean = (',num2str(mean(my_computation_time)),' \musec)');
title(title_str)

subplot(2,2,2)
HIST(my_computation_time,500)
ylabel('number of occurences')
xlabel('tick timing (\musec)')
title_str = strcat('Computation Time, std-dev = (',num2str(std(my_computation_time)),' \musec)');
title(title_str)

subplot(2,2,3)
plot(my_time,my_timestep,'o')
ylabel('tick timing (\musec)')
xlabel('time (sec)')
title_str = strcat('Sample Time, mean = (',num2str(mean(my_timestep)),' \musec)');
title(title_str)

subplot(2,2,4)
HIST(my_timestep,500)
ylabel('number of occurences')
xlabel('tick timing (\musec)')
title_str = strcat('Sample Time, std-dev = (',num2str(std(my_timestep)),' \musec)');
title(title_str)

% Figure 2: plot the three pieces of data that you recorded
figure(2)
subplot(3,1,1)
stairs(my_time,my_variable_01,'r')
xlabel('time(seconds)')
ylabel('chan 1 input')
subplot(3,1,2)
stairs(my_time,my_variable_02,'b')
xlabel('time(seconds)')
ylabel('axis 1 input')
subplot(3,1,3)
stairs(my_time,my_variable_03,'g')
xlabel('time(seconds)')
ylabel('axis 1 output')