/***********************************************************************************************/
/**************************************** queue.c **********************************************/
/***********************************************************************************************/

/***********************************************************************************************/
/*************************************** My Headers ********************************************/
/***********************************************************************************************/
#define __QUEUE
#include "queue.h"

/***********************************************************************************************/
/******** this function posts a message on one of the global FIFO queues between threads *******/
/***********************************************************************************************/
void SendMsg(int chan, unsigned int msg){
   if(chan < NUMBER_OF_QUEUES){
		if(pthread_mutex_trylock(&queuetex) == EOK){
			Thread_Msg[chan][Thread_IndexIn[chan]] = msg;
			if(++Thread_IndexIn[chan] == MSG_DEPTH) Thread_IndexIn[chan] = 0;
			pthread_mutex_unlock(&queuetex);
		}
   }
	else{
		PrintErrorMsg("Send message fail - no such queue");
	}
}
/***********************************************************************************************/
/******** this function gets a message on one of the global FIFO queues between threads ********/
/***********************************************************************************************/
unsigned int GetMsg(int chan){

	unsigned int msg = EMPTY;
   
   if(chan < NUMBER_OF_QUEUES){
      if(pthread_mutex_trylock(&queuetex) == EOK){
			if(Thread_IndexOut[chan] != Thread_IndexIn[chan]){
				msg = Thread_Msg[chan][Thread_IndexOut[chan]];
				if(++Thread_IndexOut[chan] == MSG_DEPTH) Thread_IndexOut[chan] = 0;
			}
			pthread_mutex_unlock(&queuetex);
      }
   }
	else{
     PrintErrorMsg("Get message fail - no such queue");
   }
   return msg;
}
/***********************************************************************************************/
/************************************* PurgeMsg() **********************************************/
/***********************************************************************************************/
void PurgeMsg(int chan){
   
   int i;
   
   if(chan < NUMBER_OF_QUEUES)  for (i=0;i<MSG_DEPTH;i++) Thread_Msg[chan][i] = EMPTY;
}
/***********************************************************************************************/
/************************************* InitMsg() ***********************************************/
/***********************************************************************************************/
void InitMsg(){
   
   int i,j;
   
   for(i=0;i<NUMBER_OF_QUEUES;i++){
      Thread_IndexIn[i] = 0;
      Thread_IndexOut[i] = 0;
      for(j=0;j<MSG_DEPTH;j++)	Thread_Msg[i][j] = EMPTY;
   }
}

