/***********************************************************************************************/
/**************************************** queue.h **********************************************/
/***********************************************************************************************/

#ifndef QUEUE_HEAD
#define QUEUE_HEAD

/***********************************************************************************************/
/************************************ Standard Headers *****************************************/
/***********************************************************************************************/
#include <unistd.h>
#include <sys/uio.h>
#include <sys/neutrino.h>
#include <pthread.h>

/***********************************************************************************************/
/*************************************** My Headers ********************************************/
/***********************************************************************************************/
#ifdef __QUEUE
#include "gui.h"
#endif

/* these are queue commands between the threads */
#define TERMINATE_CONTROL_THREAD 0x100
#define RELEASE_RECORDING			0x200
#define STOP_RECORDING	   		0x300
#define SUSPEND_RECORDING			0x400
#define RESUME_RECORDING			0x500
#define START_NEW_FILE				0x600
#define INCREASE_PARAMETER			0x800
#define DECREASE_PARAMETER			0x900
#define ENTER_TEST_MODE				0xa00
#define START_GUI						0xb00
#define INCREASE_FIELD				0xc00
#define DECREASE_FIELD				0xd00

#define IDLE         0x00

#define MSG_DEPTH 100
#define TO_CONTROL_MSG 0
#define TO_RECORD_MSG 1
#define TO_DISPLAY_MSG 2
#define NUMBER_OF_QUEUES 3
#define EMPTY 0xabcd

void SendMsg(int chan, unsigned int msg);
unsigned int GetMsg(int chan);
void PurgeMsg(int chan);
void InitMsg();

#ifndef __QUEUE
extern pthread_mutex_t queuetex; /* mutex for inter-thread queues */
#else
pthread_mutex_t queuetex = PTHREAD_MUTEX_INITIALIZER;/* mutex for inter-thread queues - initialized with default attributes */
unsigned int Thread_Msg[NUMBER_OF_QUEUES][MSG_DEPTH];/* used for messages between threads */
int Thread_IndexIn[NUMBER_OF_QUEUES];
int Thread_IndexOut[NUMBER_OF_QUEUES];
#endif

#ifdef __MAIN
unsigned int emsg = IDLE;
#else
extern unsigned int emsg;
#endif

#endif
